#include "ti86asm.inc"

AlivePTR        =       $8100
PosSpace        =       $8102
CurPos          =       $8104
Score           =       $8106
AliveTable      =       $8108
EndAliveTable   =       $0F     ;$810F - only need lsb to calculate address

.org _asm_exec_ram

    nop
    jp ProgStart
    .dw 0
    .dw ShellTitle

ShellTitle:
    .db "Text Racer",0

ProgStart:
 call _runindicoff
 call _clrLCD


 ld hl,$0803
 ld (_curRow),hl
 ld hl,Title
 call _puts

 ld hl,$1F2D
 ld (_penCol),hl
 ld hl,Author
 call _vputs
 call _getkey

 ;Initialize Variables
 ld bc,15               ;Data Length
 ld de,AlivePTR         ;Where to copy to
 ld hl,InitData         ;Where the initial values are stored
 ldir                   ;Copy it

 ld b,8                 ;do loop 8 times

InitMainScreenLoop:
 ld hl,String       ;\Put the initial
 call _puts         ;/"-"'s on screen

 ld hl,(CurPos)     ;\
 ld (_curRow),hl    ; \Put the initial spaces
 ld hl,Spaces       ; /(safe area) on screen
 call _puts         ;/

  call _newline
 djnz InitMainScreenLoop

Main:
 ld hl,$0007        ;\
 ld (_curRow),hl    ; \Put the "-"'s
 ld hl,String       ; /on screen
 call _puts         ;/
 
 ld hl,(CurPos)     ;\
 ld (_curRow),hl    ; \Put the spaces
 ld hl,Spaces       ; /(safe area) on screen
 call _puts         ;/

 call _newline      ;Move up one line

 ld hl,(PosSpace)   ;\
 ld (_curRow),hl    ; \Disp the Players
 ld a,LcapY         ; /position
 call _putmap       ;/

 call CheckAlive    ;see if player is within boundaries

 ld hl,(Score)      ;\
 inc hl             ; =Incriment the score
 ld (Score),hl      ;/

 ld a,r             ;get a number between 0 and $80 (mostly random)
 cp $41             ;if r is <$41
 jr c,MoveLeft      ;then move left
 jr MoveRight       ;else move right

;some of the subroutines for keypresses are here to avoid jp

DecreaseContrast:
 ld a,(_contrast)
 dec a
 jr c,RetKeyLoop
 out (2),a
 ld (_contrast),a
 jr RetKeyLoop

IncreaseContrast:
 ld a,(_contrast)
 inc a
 cp $1f
 jr z,RetKeyLoop
 out (2),a
 ld (_contrast),a
 jr RetKeyLoop
RandRet:
 ld b,25            ;Delay to slow down prog
KeyLoop:

   push bc          ;Preserve Counter between _getky calls
 call _getky
   pop bc           ;restore the counter

 cp K_MORE
 jr z,Pause

 cp K_LEFT
 jr z,LEFT

 cp K_RIGHT
 jr z,RIGHT

 cp K_PLUS
 jr z,IncreaseContrast

 cp K_MINUS
 jr z,DecreaseContrast

 cp K_EXIT
 jr z,Exit

RetKeyLoop:
 halt               ;save battery power and slow down prog
 djnz KeyLoop       ;End of the delay loop/_getky loop
 jr Main            ;delay is done so update screen

MoveLeft:                   ;move the boundary to the left
 ld a,(CurPos+1)
 dec a
 jr z,MoveRight
 ld hl,CurPos+1
 ld (hl),a
 jr RandRet

MoveRight:                  ;move the boundary to the right
 ld a,(CurPos+1)
 inc a
 cp $10
 jr z,MoveLeft
 ld hl,CurPos+1
 ld (hl),a
 jr RandRet

LEFT:
 ld a,(PosSpace+1)
 dec a
 jr z,Exit
 ld hl,PosSpace+1
 ld (hl),a
 jr RetKeyLoop

RIGHT:
 ld a,(PosSpace+1)
 inc a
 cp $13
 jr z,Exit
 ld hl,PosSpace+1
 ld (hl),a
 jr RetKeyLoop

CheckAlive:
 ld hl,(AlivePTR)

 ld a,(CurPos+1)    ;\Save bottom
 ld (hl),a          ;/row info (line 7)
;since the 256 byte boundary isnt a problem
;ignore h and only work with 8 bits (faster smaller)
 inc l
 ld d,EndAliveTable
 ld a,l
 cp d
 jr c,CheckAlive1       ;jump if pointer is valid else reset it

 ld hl,AliveTable
 ld (AlivePTR),hl
 jr CheckAlive2

CheckAlive1:
 ld (AlivePTR),hl       ;save AlivePTR

CheckAlive2:
 ld a,(hl)              ;a=(AlivePTR)
;check the left sideline
 ld hl,PosSpace+1
 ld b,(hl)              ;b=(PosSpace)
 inc b                  ;inc b so as to avoid checking for the zero flag also
 cp b
 jp p,Exit

CheckAlive3:
 ;check the right sideline
 add a,4                ;add 4 to get the right sideline
 cp b
 jr c,Exit
 ret

Pause:
 call _getkey
 jr RetKeyLoop

Exit:
 ld hl,$0007
 ld (_curRow),hl

 ld b,7
ExitLoop:
 push bc
 call _newline
 ld b,15
ExitLoop1:
 halt
 djnz ExitLoop1
 pop bc
 djnz ExitLoop

 ld hl,$0603
 ld (_curRow),hl
 ld hl,TxtEnd
 set textInverse,(iy+textflags)
 call _puts
 res textInverse,(iy+textflags)

 ld hl,$2026
 ld (_penCol),hl
 ld hl,TxtScore
 call _vputs
 ld hl,(Score)

;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
DisplayHL:
 ld c,'0'               ; save ascii value for zero
 ld de,_OP1+5           ; point to end of the buffer
 xor a                  ; zero terminate string
 ld (de),a              ; set last byte to zero
DisplayHLl:
 call _divHLby10        ; next digit
 dec de                 ; next position
 add a,c                ; convert to ascii
 ld (de),a              ; save digit
 ld a,h                 ; load upper byte
 or l                   ; check with lower byte for zero
 jr nz,DisplayHLl       ; loop
 ex de,hl               ; point to buffer
 call _vputs            ; print number
 ;ret                   ; we're done

 call _pause
 call _clrScrn
 call _homeup
 jp _jforcecmdnochar



Title:
 .db "Racer",0
Author:
 .db "By: GForce",0
String:
 .db "|^^^^^^^^^^^^^^^^^^|",0
TxtEnd:
 .db "Game Over",0
TxtScore:
 .db "Final Score:",0
Spaces:
 .db "    ",0
InitData:
 .db $08,$81,0,7,7,5,0,0,5,5,5,5,5,5,5

.end